#include "testApp.h"

float timeBetweenSmiles = 7.0;

int totalCount = 0;
bool needsSaving = false;

deque<bool> needsUpdate;
deque<ofImage*> thumbs;
int targetWidth, targetHeight;

int getMaxCount() {
	int maxWidth = ofGetWidth() / targetWidth;
	int maxHeight = ofGetHeight() / targetHeight;
	return maxWidth * maxHeight;
}

void testApp::setup() {
	ofFileUtils::makeDirectory("all");
	//ofFileUtils::makeDirectory("each");
	
	ofSetWindowTitle("Happy Things!");

	ofSetVerticalSync(true);
	ofSetFrameRate(60);
	
	targetWidth = 32;
	targetHeight = 32;
	
	ofBackground(255, 255, 255);
	
	
	//-- :) 
	
		vision.setupCamera(0, 320, 240);

		ttf.loadFont("MONACO.TTF", 48);
		ttfSmall.loadFont("MONACO.TTF", 10);

		img.allocate(320, 240, OF_IMAGE_GRAYSCALE);
		
		lastTime = ofGetElapsedTimef()- 10.0;
			
		smilePct = 0.0;
		smoothPct = 0.0;
	
	//-- :( 
}

void testApp::update() {
	
	//-- :) 
	vision.update();

	img.setFromPixels(vision.gray.getPixels(), vision.gray.width, vision.gray.height, OF_IMAGE_GRAYSCALE);
		
	ofxSmile::getSmile(img, smilePct);
	
	smoothPct *= 0.8;
	smoothPct += MAX(0, smilePct) * 0.2;
	
	if( ofGetElapsedTimef() - lastTime > timeBetweenSmiles ){
		
		if( smoothPct > 0.420 ){
			needsSaving = true;
			lastTime = ofGetElapsedTimef();
		}
							
	}
	//-- :( 


	while(thumbs.size() > getMaxCount()) {
		ofImage* cur = thumbs.front();
		thumbs.pop_front();
		needsUpdate.pop_front();
		delete cur;
	}
		
	int x = 0;
	int y = 0;
	for(int i = 0; i < thumbs.size(); i++) {
		if(needsUpdate[i]) {
			thumbs[i]->update();
			needsUpdate[i] = false;
		}	
		thumbs[i]->draw(x, y);
		x += targetWidth;
		if(x >= ofGetWidth()) {
			x = 0;
			y += targetHeight;
		}
	}
}



//--------------------------------------------------------------
void testApp::keyPressed  (int key){
    //if you need to adjust the camera properties
    if(key == 's'){
        vision.camera.videoSettings();
    }
}


void uploadFile(string filename) {
	cout << "uploading" << endl;
	string cmd = "curl -s -F file=@" + ofToDataPath(filename) + " http://kylemcdonald.net/happythings/upload/ &";
	system(cmd.c_str());
}

void testApp::globalMouseDown(float x, float y) {
}

void testApp::draw() {
	
	//-- :)
	ofSetColor(255);
	vision.color.draw(0,0);
	//-- :(
	
	if(needsSaving) {
		needsSaving = false;
		
		ofRectangle region;
		region.set(0, 0, ofGetScreenWidth(), ofGetScreenHeight());
		ofImage* cur = new ofImage();
		cur->setUseTexture(false);
		
		ofGrabGlobalScreen(*cur, region);
		string filename = "all/" + ofGetTimestampString() + ".png";
		cur->setImageType(OF_IMAGE_COLOR);
		
		ofImage videoFrame;
		videoFrame.grabScreen(0, 0,  vision.camera.getWidth(), vision.camera.getHeight());
		//videoFrame.resize(220, 165);
		
		unsigned char * dstPtr = cur->getPixels();
		int numVidPixels = vision.camera.width * vision.camera.height * 3;
		
		unsigned char * srcPtr = videoFrame.getPixels();
		
		int src, dst = 0;
		int dx, dy = 0;
		for(int y = 0; y < videoFrame.height; y++){
			for(int x = 0; x < videoFrame.width; x++){
			
				dx = x + 20;
				dy = y + 40;
			
				src = (x+y*videoFrame.width)*3;
				dst = (dx+dy*cur->width)*3;
				
				dstPtr[dst  ] *= 0.25;
				dstPtr[dst+1] *= 0.25;
				dstPtr[dst+2] *= 0.25;

				dstPtr[dst  ] += srcPtr[src  ]*0.75;
				dstPtr[dst+1] += srcPtr[src+1]*0.75;
				dstPtr[dst+2] += srcPtr[src+2]*0.75;
			}
		}
		
		cur->update();
		cur->saveImage(filename);
		uploadFile(filename);
		thumbs.push_back(cur);
		needsUpdate.push_back(true);
		
		totalCount++;
		if(totalCount % getMaxCount() == 0) {
			needsSaving = true;
		}
	}
	
	if( ofGetElapsedTimef() - lastTime < 1.0 ){
		ofSetColor(255, 255, 255, 100);
		ttf.drawString(":)", -50 + img.width/2, 30 + img.height/2);
	}

	float alpha = ofMap(  ofGetElapsedTimef() - lastTime, 1.0, 1.3, 255, 0.0, true);
	
	ofSetColor(255, 255, 255, alpha);
	ttfSmall.drawString("-> sending happy moment! :)", 10, 230);	
		
	ofSetColor(255);
	ofDrawBitmapString("smile        = " + ofToString(smilePct, 2), 20, 20);
	ofDrawBitmapString("smile smooth = " + ofToString(smoothPct, 2), 20, 36);
	
}